﻿using System.IO;
using UnityEngine;

namespace AbyssLibU
{

    /// <summary>
    /// UnityのSpriteクラスのマネージャです。
    /// （主にLuaのために）Spriteクラスに対する簡易化された操作を提供します。
    /// </summary>
    public static class SpriteManager
    {
        /// <summary>
        /// Resourcesフォルダからテクスチャデータを読み込みます。
        /// </summary>
        /// <param name="path">テクスチャデータのパスを指定します。</param>
        /// <returns>UnityのSpriteクラスを返します。</returns>
        public static Sprite LoadResources(string path)
        {
            Sprite result = Resources.Load<Sprite>(path);
            if(result==null){
                throw new FileNotFoundException("Could not find resource \"" + path + "\"");
            }
            return result;
        }
        /// <summary>
        /// 外部データからテクスチャデータを読み込みます。
        /// </summary>
        /// <param name="path">テクスチャデータのパスを指定します。</param>
        /// <param name="rect">テクスチャのRect領域を指定します。</param>
        /// <param name="pivot">Rectに対するピボット地点の相対位置を指定します。</param>
        /// <returns>UnityのSpriteクラスを返します。</returns>
        public static Sprite LoadExternalData(string path, Rect? rect = null, Vector2? pivot = null)
        {
            BinaryReader bin = new BinaryReader(new FileStream(path, FileMode.Open));
            byte[] result = bin.ReadBytes((int)bin.BaseStream.Length);
            bin.Close();
            Texture2D texture = new Texture2D(1, 1);
            texture.LoadImage(result);
            return Sprite.Create(texture, rect ?? new Rect(0, 0, texture.width, texture.height), pivot ?? new Vector2(0.5f, 0.5f));
        }
        /// <summary>
        /// テクスチャデータを読み込みます。
        /// Resourcesフォルダ⇒外部データの順に読み込みを試みます。
        /// </summary>
        /// <param name="path">テクスチャデータのパスを指定します。</param>
        /// <param name="rect">テクスチャのRect領域を指定します。Resourcesフォルダから読み込む場合は無視されます。</param>
        /// <param name="pivot">Rectに対するピボット地点の相対位置を指定します。Resourcesフォルダから読み込む場合は無視されます。</param>
        /// <returns>UnityのSpriteクラスを返します。</returns>
        public static Sprite Load(string path, Rect? rect = null, Vector2? pivot = null)
        {
            try
            {
                return LoadResources(path);
            }
            catch (FileNotFoundException)
            {
                return LoadExternalData(path, rect, pivot);
            }
        }
    }

    /// <summary>
    /// UnityのImageクラスのマネージャです。
    /// （主にLuaのために）Imageクラスに対する簡易化された操作を提供します。
    /// </summary>
    public static class ImageManager
    {
        /// <summary>
        /// ImageクラスにSpriteクラスを設定します。
        /// </summary>
        /// <param name="name">Imageクラスの名前を指定します。</param>
        /// <param name="sprite">設定するSpriteクラスを指定します。</param>
        public static void SetSprite(string name, Sprite sprite)
        {
            try
            {
                GameObject.Find(name).GetComponent<UnityEngine.UI.Image>().sprite = sprite;
            }
            catch (System.NullReferenceException)
            {
                throw new System.NullReferenceException("Could not find component \"" + name + "\"");
            }
        }
        /// <summary>
        /// ImageクラスからRectTransformクラスを取得します。
        /// </summary>
        /// <param name="name">Imageクラスの名前を指定します。</param>
        /// <returns>UnityのRectTransformクラスを返します。</returns>
        public static RectTransform GetRectTransform(string name)
        {
            try
            {
                RectTransform result = GameObject.Find(name).GetComponent<UnityEngine.UI.Image>().GetComponent<RectTransform>();
                return result;
            }
            catch (System.NullReferenceException)
            {
                throw new System.NullReferenceException("Could not find component \"" + name + "\"");
            }
        }
    }

    /// <summary>
    /// UnityのButtonクラスのマネージャです。
    /// （主にLuaのために）Buttonクラスに対する簡易化された操作を提供します。
    /// </summary>
    public static class ButtonManager
    {
        /// <summary>
        /// プレハブをインスタンス化します。
        /// </summary>
        /// <param name="name">プレハブ名を指定します。</param>
        /// <returns>インスタンスを返します。</returns>
        public static UnityEngine.UI.Button Instantiate(string name)
        {
            return (Object.Instantiate(Resources.Load<UnityEngine.UI.Button>(name)));
        }
        /// <summary>
        /// ButtonクラスからTextクラスを取得します。
        /// </summary>
        /// <param name="button">Buttonクラスを指定します。</param>
        /// <returns>Textクラスを返します。</returns>
        public static UnityEngine.UI.Text GetButtonText(UnityEngine.UI.Button button)
        {
            return (button.GetComponentInChildren<UnityEngine.UI.Text>());
        }
    }

}
